@@ -1,98 +1,82 @@
-/* -*- Mode: C -*- */
 /*======================================================================
-  FILE: icalvalue.c
-  CREATOR: eric 02 May 1999
-  
-  $Id: icalderivedvalue.c.in,v 1.15 2007-04-30 13:57:48 artcancro Exp $
-
+ FILE: icalvalue.c
+ CREATOR: eric 02 May 1999
 
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
 
  This program is free software; you can redistribute it and/or modify
- it under the terms of either: 
+ it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
     2.1, available at: http://www.fsf.org/copyleft/lesser.html
 
-  Or:
+ Or:
 
     The Mozilla Public License Version 1.0. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
-  The original code is icalvalue.c
-
-  Contributions from:
-     Graham Davison (g.m.davison@computer.org)
-
-
+ Contributions from:
+   Graham Davison (g.m.davison@computer.org)
 ======================================================================*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include "icalderivedvalue.h"
+#include "icalvalue.h"
+#include "icalvalueimpl.h"
 #include "icalerror.h"
 #include "icalmemory.h"
-#include "icalparser.h"
-#include "icalenums.h"
 
-#include "icalvalueimpl.h"
-
-#include <stdlib.h> /* for malloc */
-#include <stdio.h> /* for snprintf */
-#include <string.h> /* For memset, others */
-#include <stddef.h> /* For offsetof() macro */
 #include <errno.h>
-#include <time.h> /* for mktime */
-#include <stdlib.h> /* for atoi and atof */
-#include <limits.h> /* for SHRT_MAX */ 
-        
-#ifdef _MSC_VER
-#define strcasecmp stricmp
-#endif
+#include <stdlib.h>
+#include <string.h>
 
 struct icalvalue_impl*  icalvalue_new_impl(icalvalue_kind kind);
 
 /* This map associates each of the value types with its string
    representation */
 struct icalvalue_kind_map {
-	icalvalue_kind kind;
-	char name[20];
+        icalvalue_kind kind;
+        char name[20];
 };
 
-static const struct icalvalue_kind_map value_map[35]={
-    {ICAL_UTCOFFSET_VALUE,"UTC-OFFSET"},
-    {ICAL_TEXT_VALUE,"TEXT"},
-    {ICAL_URI_VALUE,"URI"},
-    {ICAL_TRIGGER_VALUE,"TRIGGER"},
-    {ICAL_FLOAT_VALUE,"FLOAT"},
-    {ICAL_CARLEVEL_VALUE,"CAR-LEVEL"},
+static const struct icalvalue_kind_map value_map[37]={
+    {ICAL_RECUR_VALUE,"RECUR"},
     {ICAL_STATUS_VALUE,"STATUS"},
-    {ICAL_INTEGER_VALUE,"INTEGER"},
     {ICAL_PERIOD_VALUE,"PERIOD"},
-    {ICAL_ACTION_VALUE,"ACTION"},
+    {ICAL_POLLCOMPLETION_VALUE,"POLLCOMPLETION"},
+    {ICAL_STRING_VALUE,"STRING"},
+    {ICAL_DATE_VALUE,"DATE"},
+    {ICAL_URI_VALUE,"URI"},
     {ICAL_X_VALUE,"X"},
     {ICAL_GEO_VALUE,"GEO"},
-    {ICAL_CALADDRESS_VALUE,"CAL-ADDRESS"},
-    {ICAL_STRING_VALUE,"STRING"},
-    {ICAL_QUERYLEVEL_VALUE,"QUERY-LEVEL"},
-    {ICAL_DATETIMEPERIOD_VALUE,"DATE-TIME-PERIOD"},
-    {ICAL_BOOLEAN_VALUE,"BOOLEAN"},
     {ICAL_BINARY_VALUE,"BINARY"},
+    {ICAL_TEXT_VALUE,"TEXT"},
+    {ICAL_INTEGER_VALUE,"INTEGER"},
     {ICAL_DATETIME_VALUE,"DATE-TIME"},
-    {ICAL_DURATION_VALUE,"DURATION"},
-    {ICAL_CLASS_VALUE,"CLASS"},
-    {ICAL_DATE_VALUE,"DATE"},
-    {ICAL_BUSYTYPE_VALUE,"BUSYTYPE"},
-    {ICAL_XLICCLASS_VALUE,"X-LIC-CLASS"},
-    {ICAL_REQUESTSTATUS_VALUE,"REQUEST-STATUS"},
-    {ICAL_POLLMODE_VALUE,"POLLMODE"},
     {ICAL_QUERY_VALUE,"QUERY"},
-    {ICAL_RECUR_VALUE,"RECUR"},
-    {ICAL_ATTACH_VALUE,"ATTACH"},
+    {ICAL_BOOLEAN_VALUE,"BOOLEAN"},
+    {ICAL_BUSYTYPE_VALUE,"BUSYTYPE"},
+    {ICAL_UTCOFFSET_VALUE,"UTC-OFFSET"},
+    {ICAL_TASKMODE_VALUE,"TASKMODE"},
     {ICAL_METHOD_VALUE,"METHOD"},
-    {ICAL_CMD_VALUE,"CMD"},
+    {ICAL_QUERYLEVEL_VALUE,"QUERY-LEVEL"},
+    {ICAL_REQUESTSTATUS_VALUE,"REQUEST-STATUS"},
+    {ICAL_CLASS_VALUE,"CLASS"},
+    {ICAL_POLLMODE_VALUE,"POLLMODE"},
+    {ICAL_XLICCLASS_VALUE,"X-LIC-CLASS"},
+    {ICAL_TRIGGER_VALUE,"TRIGGER"},
+    {ICAL_ATTACH_VALUE,"ATTACH"},
+    {ICAL_CALADDRESS_VALUE,"CAL-ADDRESS"},
+    {ICAL_ACTION_VALUE,"ACTION"},
     {ICAL_TRANSP_VALUE,"TRANSP"},
+    {ICAL_CMD_VALUE,"CMD"},
+    {ICAL_DATETIMEPERIOD_VALUE,"DATE-TIME-PERIOD"},
+    {ICAL_DURATION_VALUE,"DURATION"},
+    {ICAL_FLOAT_VALUE,"FLOAT"},
+    {ICAL_CARLEVEL_VALUE,"CAR-LEVEL"},
     {ICAL_NO_VALUE,""}
 };
 
@@ -477,6 +461,34 @@ struct icalperiodtype icalvalue_get_period (const icalvalue* value) {
 
 
 
+icalvalue* icalvalue_new_pollcompletion (enum icalproperty_pollcompletion v){
+   struct icalvalue_impl* impl;
+   
+   impl = icalvalue_new_impl(ICAL_POLLCOMPLETION_VALUE);
+   icalvalue_set_pollcompletion((icalvalue*)impl,v);
+   return (icalvalue*)impl;
+}
+void icalvalue_set_pollcompletion(icalvalue* value, enum icalproperty_pollcompletion v) {
+    struct icalvalue_impl* impl; 
+    icalerror_check_arg_rv( (value!=0),"value");
+    
+    icalerror_check_value_type(value, ICAL_POLLCOMPLETION_VALUE);
+    impl = (struct icalvalue_impl*)value;
+
+
+    impl->data.v_enum = v; 
+
+    icalvalue_reset_kind(impl);
+}
+enum icalproperty_pollcompletion icalvalue_get_pollcompletion (const icalvalue* value) {
+
+    icalerror_check_arg ((value!=0),"value");
+    icalerror_check_value_type (value, ICAL_POLLCOMPLETION_VALUE);
+    return ((struct icalvalue_impl*)value)->data.v_enum;
+}
+
+
+
 icalvalue* icalvalue_new_pollmode (enum icalproperty_pollmode v){
    struct icalvalue_impl* impl;
    
@@ -661,6 +673,34 @@ const char* icalvalue_get_string (const icalvalue* value) {
 
 
 
+icalvalue* icalvalue_new_taskmode (enum icalproperty_taskmode v){
+   struct icalvalue_impl* impl;
+   
+   impl = icalvalue_new_impl(ICAL_TASKMODE_VALUE);
+   icalvalue_set_taskmode((icalvalue*)impl,v);
+   return (icalvalue*)impl;
+}
+void icalvalue_set_taskmode(icalvalue* value, enum icalproperty_taskmode v) {
+    struct icalvalue_impl* impl; 
+    icalerror_check_arg_rv( (value!=0),"value");
+    
+    icalerror_check_value_type(value, ICAL_TASKMODE_VALUE);
+    impl = (struct icalvalue_impl*)value;
+
+
+    impl->data.v_enum = v; 
+
+    icalvalue_reset_kind(impl);
+}
+enum icalproperty_taskmode icalvalue_get_taskmode (const icalvalue* value) {
+
+    icalerror_check_arg ((value!=0),"value");
+    icalerror_check_value_type (value, ICAL_TASKMODE_VALUE);
+    return ((struct icalvalue_impl*)value)->data.v_enum;
+}
+
+
+
 icalvalue* icalvalue_new_text (const char* v){
    struct icalvalue_impl* impl;
    icalerror_check_arg_rz( (v!=0),"v");
@@ -821,20 +861,20 @@ int icalvalue_kind_is_valid(const icalvalue_kind kind)
     int i = 0;
     do {
       if (value_map[i].kind == kind)
-	return 1;
+        return 1;
     } while (value_map[i++].kind != ICAL_NO_VALUE);
 
     return 0;
-}  
+}
 
 const char* icalvalue_kind_to_string(const icalvalue_kind kind)
 {
     int i;
 
     for (i=0; value_map[i].kind != ICAL_NO_VALUE; i++) {
-	if (value_map[i].kind == kind) {
-	    return value_map[i].name;
-	}
+        if (value_map[i].kind == kind) {
+            return value_map[i].name;
+        }
     }
 
     return 0;
@@ -845,9 +885,9 @@ icalvalue_kind icalvalue_string_to_kind(const char* str)
     int i;
 
     for (i=0; value_map[i].kind != ICAL_NO_VALUE; i++) {
-	if (strcasecmp(value_map[i].name,str) == 0) {
-	    return value_map[i].kind;
-	}
+        if (strcasecmp(value_map[i].name,str) == 0) {
+            return value_map[i].kind;
+        }
     }
 
     return  value_map[i].kind;
@@ -874,7 +914,7 @@ void icalvalue_set_x(icalvalue* impl, const char* v) {
     if (impl->x_value == 0){
       errno = ENOMEM;
     }
- 
+
  }
 const char* icalvalue_get_x(const icalvalue* value) {
 
@@ -888,7 +928,7 @@ icalvalue*
 icalvalue_new_recur (struct icalrecurrencetype v)
 {
    struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_RECUR_VALUE);
-    
+
    icalvalue_set_recur((icalvalue*)impl,v);
 
    return (icalvalue*)impl;
@@ -901,19 +941,19 @@ icalvalue_set_recur(icalvalue* impl, struct icalrecurrencetype v)
     icalerror_check_value_type(value, ICAL_RECUR_VALUE);
 
     if (impl->data.v_recur != 0){
-	free(impl->data.v_recur);
-	impl->data.v_recur = 0;
+        free(impl->data.v_recur);
+        impl->data.v_recur = 0;
     }
 
     impl->data.v_recur = malloc(sizeof(struct icalrecurrencetype));
 
     if (impl->data.v_recur == 0){
-	icalerror_set_errno(ICAL_NEWFAILED_ERROR);
-	return;
+        icalerror_set_errno(ICAL_NEWFAILED_ERROR);
+        return;
     } else {
-	memcpy(impl->data.v_recur, &v, sizeof(struct icalrecurrencetype));
+        memcpy(impl->data.v_recur, &v, sizeof(struct icalrecurrencetype));
     }
-	       
+
 }
 
 struct icalrecurrencetype
@@ -924,7 +964,7 @@ icalvalue_get_recur(const icalvalue* value)
 
     icalerror_check_arg_rx( (value!=0),"value", rt);
     icalerror_check_value_type(value, ICAL_RECUR_VALUE);
-  
+
     return *(value->data.v_recur);
 }
 
@@ -935,7 +975,7 @@ icalvalue*
 icalvalue_new_trigger (struct icaltriggertype v)
 {
    struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_TRIGGER_VALUE);
- 
+
    icalvalue_set_trigger((icalvalue*)impl,v);
 
    return (icalvalue*)impl;
@@ -945,7 +985,7 @@ void
 icalvalue_set_trigger(icalvalue* value, struct icaltriggertype v)
 {
     icalerror_check_arg_rv( (value!=0),"value");
-    
+
    if(!icaltime_is_null_time(v.time)){
        icalvalue_set_datetime(value,v.time);
        value->kind = ICAL_DATETIME_VALUE;
@@ -955,40 +995,32 @@ icalvalue_set_trigger(icalvalue* value, struct icaltriggertype v)
    }
 }
 
-struct icaltriggertype
-icalvalue_get_trigger(const icalvalue* impl)
+struct icaltriggertype icalvalue_get_trigger(const icalvalue* impl)
 {
     struct icaltriggertype tr;
 
     tr.duration = icaldurationtype_from_int(0);
     tr.time = icaltime_null_time();
 
-    icalerror_check_arg_rx( (impl!=0),"value", tr);
+    icalerror_check_arg_rx((impl != 0),"value", tr);
 
-    if(impl) {
-      if(impl->kind == ICAL_DATETIME_VALUE){
-	tr.duration = icaldurationtype_from_int(0);
-	tr.time = impl->data.v_time;
-      } else if(impl->kind == ICAL_DURATION_VALUE){
-	tr.time = icaltime_null_time();
-	tr.duration = impl->data.v_duration;
-      } else {
-	tr.duration = icaldurationtype_from_int(0);
-	tr.time = icaltime_null_time();
-	icalerror_set_errno(ICAL_BADARG_ERROR);
-      }
+    if (impl->kind == ICAL_DATETIME_VALUE) {
+        tr.duration = icaldurationtype_from_int(0);
+        tr.time = impl->data.v_time;
+    } else if (impl->kind == ICAL_DURATION_VALUE) {
+        tr.time = icaltime_null_time();
+        tr.duration = impl->data.v_duration;
     } else {
-      tr.duration = icaldurationtype_from_int(0);
-      tr.time = icaltime_null_time();
-      icalerror_set_errno(ICAL_BADARG_ERROR);
+        tr.duration = icaldurationtype_from_int(0);
+        tr.time = icaltime_null_time();
+        icalerror_set_errno(ICAL_BADARG_ERROR);
     }
-    
+
     return tr;
 }
 
-icalvalue*
-icalvalue_new_datetime (struct icaltimetype v){
-
+icalvalue *icalvalue_new_datetime (struct icaltimetype v)
+{
    struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_DATETIME_VALUE);
    icalvalue_set_datetime((icalvalue*)impl,v);
    return (icalvalue*)impl;
@@ -1034,25 +1066,25 @@ void
 icalvalue_set_datetimeperiod(icalvalue* impl, struct icaldatetimeperiodtype v)
 {
     icalerror_check_arg_rv( (impl!=0),"value");
-    
+
     icalerror_check_value_type(value, ICAL_DATETIMEPERIOD_VALUE);
 
     if(!icaltime_is_null_time(v.time)){
-	if(!icaltime_is_valid_time(v.time)){
-	    icalerror_set_errno(ICAL_BADARG_ERROR);
-	    return;
-	}
-	impl->kind = ICAL_DATETIME_VALUE;
-	icalvalue_set_datetime(impl,v.time);
+        if(!icaltime_is_valid_time(v.time)){
+            icalerror_set_errno(ICAL_BADARG_ERROR);
+            return;
+        }
+        impl->kind = ICAL_DATETIME_VALUE;
+        icalvalue_set_datetime(impl,v.time);
     } else if (!icalperiodtype_is_null_period(v.period)) {
-	if(!icalperiodtype_is_valid_period(v.period)){
-	    icalerror_set_errno(ICAL_BADARG_ERROR);
-	    return;
-	}
-	impl->kind = ICAL_PERIOD_VALUE;
-	icalvalue_set_period(impl,v.period);
+        if(!icalperiodtype_is_valid_period(v.period)){
+            icalerror_set_errno(ICAL_BADARG_ERROR);
+            return;
+        }
+        impl->kind = ICAL_PERIOD_VALUE;
+        icalvalue_set_period(impl,v.period);
     } else {
-	icalerror_set_errno(ICAL_BADARG_ERROR);
+        icalerror_set_errno(ICAL_BADARG_ERROR);
     }
 }
 
@@ -1067,23 +1099,17 @@ icalvalue_get_datetimeperiod(const icalvalue* impl)
   icalerror_check_arg_rx( (impl!=0),"value", dtp);
   icalerror_check_value_type(value, ICAL_DATETIMEPERIOD_VALUE);
 
-  if(impl) {
-    if( impl->kind == ICAL_DATETIME_VALUE || impl->kind == ICAL_DATE_VALUE ){
+  if( impl->kind == ICAL_DATETIME_VALUE || impl->kind == ICAL_DATE_VALUE ){
       dtp.period = icalperiodtype_null_period();
       dtp.time = impl->data.v_time;
-    } else if(impl->kind == ICAL_PERIOD_VALUE) {
+  } else if(impl->kind == ICAL_PERIOD_VALUE) {
       dtp.period = impl->data.v_period;
       dtp.time = icaltime_null_time();
-    } else {
+  } else {
       dtp.period = icalperiodtype_null_period();
       dtp.time = icaltime_null_time();
       icalerror_set_errno(ICAL_BADARG_ERROR);
-    }	
-  } else {
-    dtp.period = icalperiodtype_null_period();
-    dtp.time = icaltime_null_time();
-    icalerror_set_errno(ICAL_BADARG_ERROR);
-  }	
+  }
 
   return dtp;
 }
@@ -1163,8 +1189,8 @@ icalvalue_new_attach (icalattach *attach)
 
     impl = icalvalue_new_impl (ICAL_ATTACH_VALUE);
     if (!impl) {
-	errno = ENOMEM;
-	return NULL;
+        errno = ENOMEM;
+        return NULL;
     }
 
     icalvalue_set_attach ((icalvalue *) impl, attach);
@@ -1175,18 +1201,18 @@ void
 icalvalue_set_attach (icalvalue *value, icalattach *attach)
 {
     struct icalvalue_impl *impl;
- 
+
     icalerror_check_arg_rv ((value != NULL), "value");
     icalerror_check_value_type (value, ICAL_ATTACH_VALUE);
     icalerror_check_arg_rv ((attach != NULL), "attach");
-  
+
     impl = (struct icalvalue_impl *) value;
- 
+
     icalattach_ref (attach);
 
     if (impl->data.v_attach)
-	icalattach_unref (impl->data.v_attach);
-  
+        icalattach_unref (impl->data.v_attach);
+
     impl->data.v_attach = attach;
 }
 
@@ -1195,7 +1221,7 @@ icalvalue_get_attach (const icalvalue *value)
 {
     icalerror_check_arg_rz ((value != NULL), "value");
     icalerror_check_value_type (value, ICAL_ATTACH_VALUE);
-    
+
     return value->data.v_attach;
 }
 
